#! /home/user/.venv/bin/python

import asyncio
from shazamio import Shazam, Serialize
from pathlib import Path
import click
from dotenv import load_dotenv
import os
from tqdm import tqdm
from typing import Optional
import sys
from collections import namedtuple
from joblib import Memory
from pydantic import BaseModel, Field

base_path = Path(__file__).parent.parent
memory = Memory(base_path / "./.cache", verbose=0)


class Match(BaseModel):
    old: Path
    new: Optional[Path] = Field(default=None)
    title: Optional[str] = Field(default=None)
    artist: Optional[str] = Field(default=None)


@click.group()
def cli():
    pass


@cli.command()
@click.option(
    "-s",
    "--song",
    required=True,
    type=click.Path(exists=True, dir_okay=False, path_type=Path),
)
def rename(song: Path):
    change = get_match(song)
    if change and change.new is not None:
        prompt_rename(change)


def songs(root: Path):
    seasons = ["winter", "spring", "summer", "fall"]
    pattern = "{}_*/*.mp3"
    matches = []
    for season in seasons:
        for file_path in root.rglob(pattern.format(season)):
            matches.append(file_path)

    def small(f: Path, max_bytes: int = 10_000_000):
        return f.stat().st_size < max_bytes

    return filter(small, matches)


def needs_rename(song: Match):
    if song.new is None:
        print(f"\nno match: {song.old.name}\n")
        return False
    if song.old.name == song.new.name:
        print(f"\nname correct: {song.old.name}\n")
        return False
    return True


@cli.command("main")
def main():
    root = Path(os.getenv("MUSIC_PATH", "~/Music"))
    if not root.exists():
        raise ValueError(f"{str(root)} does not exist")

    renames = []
    for song in tqdm(songs(root), position=0, leave=True):
        rename = get_match(song)
        if rename and rename.new is not None:
            renames.append(rename)

    for song in filter(needs_rename, renames):
        prompt_rename(song)

    sys.exit(127)


def prompt_rename(song: Match):
    old = song.old
    answer = input(
        f'\nrename: "{old.name}"\nto: "{song.new.name}"?\n[y[es]/n[o]/m[anual]] > '
    )
    if answer in ["n", "N"]:
        print(f"\nskipping: {old.name}\n")
        return
    if answer in ["m"]:
        manual = input("\nnew name: ")
        new = old.parent / manual
        assert (
            new.suffix == old.suffix
        ), f"filetype changed: {old.suffix} -> {new.suffix}"
    else:
        new = song.new
    try:
        old.rename(new)
        from mutagen.easyid3 import EasyID3

        tags = EasyID3(new)
        tags["title"] = song.title
        tags["artist"] = song.artist
        tags.save()
    except FileNotFoundError as e:
        print(f"error renaming: {e}")
    print(f'old: "{old}"\nnew: "{new}"')
    return


@memory.cache
def get_match(original: Path) -> Match:
    loop = asyncio.get_event_loop()
    shazam = Shazam()
    try:
        out = loop.run_until_complete(shazam.recognize(str(original)))
        serialized = Serialize.track(out["track"])
        rename = f"{serialized.title} - {serialized.subtitle}{original.suffix}"
        new = Path(original.parent) / rename
        return Match(
            old=original, new=new, title=serialized.title, artist=serialized.subtitle
        )
    except:
        return Match(old=original, new=None)


if __name__ == "__main__":
    env = Path(__file__).parent.parent / ".env"
    if not load_dotenv(env):
        raise ValueError(".env not found: {env}")
    cli()
